
' Dark Spelunker! v1.0
' (C) 2007 i-TECH and Kristopher Windsor

'This program only includes the iTECH library for the font and menu features (some files in the itech folder are unused).
'Graphics are done via two layers: the cave is on the bottom, and the black layer on top moves and has changing transparency
'Methods are a bit odd to adjust for the graphics; it may be confusing if additional features are added
'Trail uses an excesses of swap commands
'Display is slow because of screenlock, screensync, sleep, and display_graphic (for blending)
'NOTE: change screenx to get a better frame rate

#define use_font
'#define use_avicapture
#define use_highscores
#define use_imenu
#include "itech\main.bi"

Const screenx = 480, screeny = 320, game_speed = 2, uplayer_trail = 50

Dim Shared As Integer cave_spot_update, cave_spot_position, cave_spot_brown_offset
Dim Shared As Integer player_died, player_trail(0 To uplayer_trail), fc
Dim Shared As Double cave_space_top, cave_space_velocity, cave_space_height
Dim Shared As Double player_position, player_velocity, st
Dim Shared As Uinteger Ptr cave_brown_graphic, cave_black_graphic, display_graphic

Sub cave_start
  Dim As Integer c, r, x, y
  display_graphic = imagecreate(screenx, screeny)
  cave_brown_graphic = imagecreate(screenx, screeny)
  For a As Integer = 1 To 800
    c = Int(Rnd * 150) + 30
    c = Rgb(c + 32, c + 16, c) 'sepia brown (originally Rgb(Int(Rnd * 50) + 50, Int(Rnd * 30) + 30, Int(Rnd * 20) + 10))
    r = Int(Rnd * 4) + 4
    x = Int(Rnd * screenx)
    y = Int(Rnd * screeny)
    Circle cave_brown_graphic, (x, y), r, c,,, 1, f
    If x - r < 0 Then Circle cave_brown_graphic, (x + screenx, y), r, c,,,, f
    If x + r >= screenx Then Circle cave_brown_graphic, (x - screenx, y), r, c,,,, f
  Next a
  cave_black_graphic = imagecreate(screenx, screeny)
End Sub

Sub cave_reset
  cave_spot_update = 0 'the column of the graphic that is redrawn and comes onto the screen
  cave_spot_position = Int(screenx * .25) 'coords of the player on the black graphic
  cave_spot_brown_offset = 0 'the brown graphic moves faster
  cave_space_top = screeny * .25 'highest place player can be
  cave_space_velocity = 1 'the cave roof starts getting lower
  cave_space_height = screeny * .5 'half of the cave is empty
  Line cave_black_graphic, (0, 0) - (screenx - 1, screeny - 1), &HFF000000, bf
End Sub

Sub cave_move
  For a As Integer = 1 To game_speed
    'advance one pixel
    cave_spot_update += 1
    If cave_spot_update = screenx Then cave_spot_update = 0
    cave_spot_position += 1
    If cave_spot_position = screenx Then cave_spot_position = 0
    cave_spot_brown_offset = (cave_spot_brown_offset + 2) Mod screenx
    'make empty space column move up or down
    If Int(Rnd * 80) = 0 Then
      cave_space_velocity = Rnd * Sgn(cave_space_velocity) * 2
      cave_space_height -= 2
    End If
    If cave_space_top < 10 Or cave_space_top + cave_space_height >= screeny - 10 Then cave_space_velocity = -cave_space_velocity
    cave_space_top += cave_space_velocity
    'update graphic
    Line cave_black_graphic, (cave_spot_update, 0) - Step(0, screeny - 1), &HFFFF00FF
    Line cave_black_graphic, (cave_spot_update, cave_space_top) - Step(0, cave_space_height), &HFF000000
  Next a
End Sub

Sub cave_display
  Put display_graphic, (-cave_spot_brown_offset - 1, 0), cave_brown_graphic, Pset
  Put display_graphic, (screenx - cave_spot_brown_offset - 1, 0), cave_brown_graphic, Pset
  Put display_graphic, (-cave_spot_update - 1, 0), cave_black_graphic, trans
  Put display_graphic, (screenx - cave_spot_update - 1, 0), cave_black_graphic, trans
End Sub

Sub cave_finish
  imagedestroy(display_graphic)
  imagedestroy(cave_brown_graphic)
  imagedestroy(cave_black_graphic)
End Sub

Sub player_reset
  player_position = screeny * .5
  player_velocity = -8
  player_died = false
  For a As Integer = 0 To uplayer_trail
    player_trail(a) = player_position
  Next a
End Sub

Sub player_move
  If Multikey(&H39) Then
    player_velocity -= .6
  Else
    player_velocity += .6
  End If
  player_position += player_velocity
  If player_position < 0 Or player_position >= screeny Then
    player_died = true
  Else
    If Point(Abs(cave_spot_position) Mod screenx, player_position, cave_black_graphic) = &HFFFF00FF Then player_died = true
  End If
  For a As Integer = uplayer_trail To 1 Step -1
    Swap player_trail(a), player_trail(a - 1)
  Next a
  player_trail(0) = player_position
End Sub

Sub player_display
  For a As Integer = 0 To uplayer_trail
    Circle display_graphic, (screenx * .25 - a * game_speed, player_trail(a)), 8, &HFFFFFFFF,,, 1, f
  Next a
End Sub

Sub game_start
  start screenx, screeny
  cave_start
  font_start "font.bmp"
  highscore_start "darkspelunker_highscores.dat"
  #ifdef use_avicapture
    avi_start "DarkSpelunker.avi", screenx, screeny, 25
  #Endif
End Sub

Sub game_reset
  cave_reset
  player_reset
  fc = 0
  st = Timer
End Sub

Sub game_move
  cave_move
  player_move
End Sub

Sub game_display
  Dim As Integer fps = Int(fc / (Timer - st))
  cave_display
  player_display
  Screenlock
  Put (0, 0), display_graphic, alpha, 50
  font_display " [Time " & fc & "] [Level " & Int((screeny * .5 - cave_space_height) * (200 / screeny)) & "] ", font_center, 4,, true
  #ifndef use_avicapture
    If fc > 100 And fps < 20 Then font_display " [Too Slow] [FPS " & fps & "]", font_center, screeny - font_size_y - 5,, true
  #Endif
  screensync
  Screenunlock
End Sub

Sub game_play
  Dim As Integer a, b, quit

  Do
    fc += 1
    If key = Chr(27) Then quit = (imenu("Quit?", "No:::Yes") = 2)

    game_move
    game_display
    #ifdef use_avicapture
      If fc Mod 2 Then avi_capture
    #Endif
    Sleep 20
    key = Inkey
  Loop Until quit Or player_died
End Sub

Sub game_finish
  finish
  cave_finish
End Sub

game_start
Do
  game_reset
  game_play
  Do: Sleep 50: Loop Until Inkey = ""
  highscore_add Int((screeny * .5 - cave_space_height) * (200 / screeny))
  highscore_display
Loop Until imenu("Play Again?", "Yes:::No") = 2
game_finish
System
